$title	GTAP6DATA.GMS	Read a GTAP 6 dataset

*	Note by Shiro Takeda: This gtap6data.gms is originally provided in
*	GTAPinGAMS created by Thomas F. Rutherford altough its content is
*	slightly modified by me.

*	------ The original header starts here. ------

*	Initial creation data: 2 August 2005 (tfr)

*	Updates:

*	3 August 2005 - Introduced sets define sector-specific factors 
*	sf(f) and mobile factors mf(f) on the basis of elasticity 
*	etrae(f) which is read from default.prm.

*	Extract files from the dataset, retrieve the GDX file
*	and then cleanup:

*	----------------------------------------------------------------------
*	Import data:

$call 'har2gdx .\basedata.har'
$call 'har2gdx .\default.prm'
$call 'har2gdx .\gsdvole.har'

$label start
set	i(*)	Goods
	f(*)	Factors
	r(*)	Regions
;
*	----------------------------------------------------------------------
*	Load and declare sets:
display "com: Load and declare sets:";
$gdxin '.\basedata.gdx'
$load r=reg f=endw_comm i=prod_comm

*	----------------------------------------------------------------------
*	Set declaration:
set	cgd(i)	Investment good /cgds/,
	rnum(r)	Numeraire region,
	sf(f)	Sluggish primary factors (sector-specific)
	mf(f)	Mobile primary factors
	src	Sources /domestic, imported/,
	ec	Energy goods /
		ecoa	Coal,
		eoil	Crude oil,
		egas	Natural gas,
		ep_c	Refined oil products,
		eely	Electricity   /;
alias (r,s), (i,j,jj), (f,ff);
display r, i, f;

*	----------------------------------------------------------------------
*	Declare parameters:
display "com: Declare parameters:";
parameters
	vdga(i,r)	Government - domestic purchases at agents' prices,
	viga(i,r)	Government - imports at agents' prices,
	vdgm(i,r)	Government - domestic purchases at market prices,
	vigm(i,r)	Government - imports at market prices,
	vdpa(i,r)	Private households - domestic purchases at agents' prices,
	vipa(i,r)	Private households - imports at agents' prices,
	vdpm(i,r)	Private households - domestic purchases at market prices,
	vipm(i,r)	Private households - imports at market prices
	evoa(f,r)	Endowments - output at agents' prices,
	evfa(f,j,r)	Endowments - firms' purchases at agents' prices,
	vfm(f,j,r)	Endowments - Firms' purchases at market prices,
	vdfa(i,j,r)	Intermediates - firms' domestic purchases at agents' prices,
	vifa(i,j,r)	Intermediates - Firms' imports at agents' prices,
	vdfm(i,j,r)	Intermediates - firms' domestic purchases at market prices,
	vifm(i,j,r)	Intermediates - firms' imports at market prices,
	vims(i,r,s)	Trade - bilateral imports at market prices,
	viws(i,r,s)	Trade - bilateral imports at world prices,
	vxmd(i,r,s)	Trade - bilateral exports at market prices,
	vxwd(i,r,s)	Trade - bilateral exports at world prices,
	vst(i,r)	Trade - exports for international transportation
	vtwr(i,j,r,s)	Trade - Margins for international transportation at world prices
	ftrv(f,j,r)	Taxes - factor employment tax revenue,

	fbep(f,j,r)	Protection - factor-based subsidies,
	isep(i,j,r,src) Protection - intermediate input subsidies,
	osep(i,r)	Protection - ordinary output subsidies,
	adrv(i,r,s)	Protection - anti-dumping duty
	tfrv(i,r,s)	Protection - ordinary import duty
	purv(i,r,s)	Protection - price undertaking export tax equivalent,
	vrrv(i,r,s)	Protection - VER export tax equivalent
	mfrv(i,r,s)	Protection - MFA export tax equivalent
	xtrv(i,r,s)	Protection - ordinary export tax

	vdep(r)		Value of depreciation
;

$load vdga viga vdgm vigm vdpa vipa vdpm vipm evoa evfa vfm vdfa
$load vifa vdfm vifm vims viws vxmd vxwd vst vtwr fbep isep
$load osep adrv tfrv purv vrrv mfrv xtrv
$load vdep

* display vdga, viga, vdgm, vigm, vdpa, vipa, vdpm, vipm, evoa, evfa, vfm, vdfa;
* display vifa, vdfm, vifm, vims, viws, vxmd, vxwd, vst, vtwr, fbep, isep;
* display osep, adrv, tfrv, purv, vrrv, mfrv, xtrv;

$load ftrv
$if NOT errorfree $goto missingftrv
$goto allread

$label missingftrv
display "ftrv not found in HAR file. Value set to zero.";
$log *** ftrv not found in HAR file. Value set to zero.
$log *** Ignore the following error message and warning
$clearerror
ftrv(f,j,r) = 0;

$label allread
$gdxin

*	----------------------------------------------------------------------
*	Convert scale:
display "com: Convert scale:";

*	Devide values by one million.
scalar scalefactor /1e-6/;
vdga(i,r) = scalefactor * vdga(i,r);
viga(i,r) = scalefactor * viga(i,r);
vdgm(i,r) = scalefactor * vdgm(i,r);
vigm(i,r) = scalefactor * vigm(i,r);
vdpa(i,r) = scalefactor * vdpa(i,r);
vipa(i,r) = scalefactor * vipa(i,r);
vdpm(i,r) = scalefactor * vdpm(i,r);
vipm(i,r) = scalefactor * vipm(i,r);
evoa(f,r) = scalefactor * evoa(f,r);
evfa(f,j,r) = scalefactor * evfa(f,j,r);
vfm(f,j,r) = scalefactor * vfm(f,j,r);
vdfa(i,j,r) = scalefactor * vdfa(i,j,r);
vifa(i,j,r) = scalefactor * vifa(i,j,r);
vdfm(i,j,r) = scalefactor * vdfm(i,j,r);
vifm(i,j,r) = scalefactor * vifm(i,j,r);
vims(i,r,s) = scalefactor * vims(i,r,s);
viws(i,r,s) = scalefactor * viws(i,r,s);
vxmd(i,r,s) = scalefactor * vxmd(i,r,s);
vxwd(i,r,s) = scalefactor * vxwd(i,r,s);
vst(i,r) = scalefactor * vst(i,r);
vtwr(i,j,r,s) = scalefactor * vtwr(i,j,r,s);
ftrv(f,j,r) = scalefactor * ftrv(f,j,r);
fbep(f,j,r) = scalefactor * fbep(f,j,r);
isep(i,j,r,src) = scalefactor * isep(i,j,r,src);
osep(i,r) = scalefactor * osep(i,r);
adrv(i,r,s) = scalefactor * adrv(i,r,s);
tfrv(i,r,s) = scalefactor * tfrv(i,r,s);
purv(i,r,s) = scalefactor * purv(i,r,s);
vrrv(i,r,s) = scalefactor * vrrv(i,r,s);
mfrv(i,r,s) = scalefactor * mfrv(i,r,s);
xtrv(i,r,s) = scalefactor * xtrv(i,r,s);
vdep(r) = scalefactor * vdep(r);

*	----------------------------------------------------------------------
*	Energy data:
display "com: Energy data:";
parameter
	evf(ec,i,r)	Volume of input purchases by firms (mtoe)
	evh(ec,r)	Volume of purchases by households (mtoe)
	evt(ec,r,r)	Volume of bilateral trade (mtoe)
	evq(ec,r)	Energy production (mtoe);

$gdxin '.\gsdvole.gdx'
$load evf evh evt
evq(ec,r) = sum(i, evf(ec,i,r)) + evh(ec,r) + sum(s, evt(ec,r,s)-evt(ec,s,r));
* display evq;

*	----------------------------------------------------------------------
*	Derive tax rates:
display "com: Derive tax rates:";

*	Declare some intermediate arrays which are required to
*	evaluate tax rates:
parameter	vdm(i,r)	Aggregate demand for domestic output,
		vom(i,r)	Total supply at market prices,
		voa(i,r)	Output at producer prices;

vdm(i,r) = vdpm(i,r) + vdgm(i,r) + sum(j, vdfm(i,j,r));
vom(i,r) = vdm(i,r) + sum(s, vxmd(i,r,s)) + vst(i,r);
voa(j,r) = sum(f, evfa(f,j,r)) + sum(i, vdfa(i,j,r) + vifa(i,j,r));
vom(cgd,r) = voa(cgd,r);
* display vdm, vom, voa, vom;

parameter
	rto(i,r)	Output (or income) subsidy rates
	rtf(f,j,r)	Primary factor and commodity rates taxes 
	rtpd(i,r)	Private domestic consumption taxes
	rtpi(i,r)	Private import consumption tax rates
	rtgd(i,r)	Government domestic rates
	rtgi(i,r)	Government import tax rates
	rtfd(i,j,r)	Firms domestic tax rates
	rtfi(i,j,r)	Firms' import tax rates
	rtxs(i,r,s)	Export subsidy rates
	rtms(i,r,s)	Import taxes rates;

rto(i,r) = 1 - voa(i,r)/vom(i,r);
rtf(f,j,r)$vfm(f,j,r)  = evfa(f,j,r)/vfm(f,j,r) - 1;
rtpd(i,r)$vdpm(i,r)   = vdpa(i,r)/vdpm(i,r) - 1;
rtpi(i,r)$vipm(i,r)   = vipa(i,r)/vipm(i,r) - 1;
rtgd(i,r)$vdgm(i,r)   = vdga(i,r)/vdgm(i,r) - 1;
rtgi(i,r)$vigm(i,r)   = viga(i,r)/vigm(i,r) - 1;
rtfd(i,j,r)$vdfm(i,j,r) = vdfa(i,j,r)/vdfm(i,j,r) - 1;
rtfi(i,j,r)$vifm(i,j,r) = vifa(i,j,r)/vifm(i,j,r) - 1;
rtxs(i,r,s)$vxwd(i,r,s) = 1-vxwd(i,r,s)/vxmd(i,r,s);
rtms(i,r,s)$viws(i,r,s) = vims(i,r,s)/viws(i,r,s) - 1;
* display rto, rtf, rtpd, rtpi, rtgd, rtgi, rtfd, rtfi, rtxs, rtms;

*	Store benchmark tax rates to construct reference prices:
parameter
	rtf0(f,j,r)	Primary factor and commodity rates taxes
	rtpd0(i,r)	Private domestic consumption taxes
	rtpi0(i,r)	Private import consumption tax rates
	rtgd0(i,r)	Government domestic rates
	rtgi0(i,r)	Government import tax rates
	rtfd0(i,j,r)	Firms domestic tax rates
	rtfi0(i,j,r)	Firms' import tax rates
	rtxs0(i,r,s)	Export subsidy rates
	rtms0(i,r,s)	Import taxes rates;

rtf0(f,j,r) = rtf(f,j,r);
rtpd0(i,r) = rtpd(i,r);
rtpi0(i,r) = rtpi(i,r);
rtgd0(i,r) = rtgd(i,r);
rtgi0(i,r) = rtgi(i,r);
rtfd0(i,j,r) = rtfd(i,j,r);
rtfi0(i,j,r) = rtfi(i,j,r);
rtxs0(i,r,s) = rtxs(i,r,s);
rtms0(i,r,s) = rtms(i,r,s);

parameter	pvxmd(i,s,r)	Import price (power of benchmark tariff)
		pvtwr(i,s,r)	Import price for transport services;

pvxmd(i,s,r) = (1+rtms0(i,s,r)) * (1-rtxs0(i,s,r));
pvtwr(i,s,r) = 1+rtms0(i,s,r);

*	----------------------------------------------------------------------
*	Declare parameters:
display "com: Declare parameters:";

parameter
	vtw(j)		Aggregate international transportation services,
	vpm(r)		Aggregate private demand,
	vgm(r)		Aggregate public demand,
	vim(i,r)	Aggregate imports,
	evom(f,r)	Aggregate factor endowment at market prices,
	vb(*)		Current account balance;

vtw(j) = sum(r, vst(j,r));
vpm(r) = sum(i, vdpm(i,r)*(1+rtpd0(i,r)) + vipm(i,r)*(1+rtpi0(i,r)));
vgm(r) = sum(i, vdgm(i,r)*(1+rtgd0(i,r)) + vigm(i,r)*(1+rtgi0(i,r)));
vim(i,r) = vipm(i,r) + vigm(i,r) + sum(j, vifm(i,j,r));
evom(f,r) = sum(j, vfm(f,j,r));
vb(r) = vpm(r) + vgm(r) + sum(cgd,vom(cgd,r)) 
	- sum(f, evom(f,r))
	- sum(j,  vom(j,r)*rto(j,r))
	- sum(j,  sum(i, vdfm(i,j,r)*rtfd(i,j,r) + vifm(i,j,r)*rtfi(i,j,r)))
	- sum(j,  sum(f, vfm(f,j,r)*rtf(f,j,r)))
	- sum(i, vdpm(i,r)*rtpd(i,r) + vipm(i,r)*rtpi(i,r))
	- sum(i, vdgm(i,r)*rtgd(i,r) + vigm(i,r)*rtgi(i,r))
	- sum((i,s), rtms(i,s,r) *  (vxmd(i,s,r) * (1-rtxs(i,s,r)) + sum(j,vtwr(j,i,s,r))))
	+ sum((i,s), rtxs(i,r,s) * vxmd(i,r,s));

vb("chksum") = sum(r, vb(r));
* display vb;

*	----------------------------------------------------------------------
*	Import EOS parameters:
display "com: Import EOS parameters:";
parameter
	esubd(i)	Elasticity of substitution (M versus D),
	esubva(j)	Elasticity of substitution between factors
	esubm(i)	Intra-import elasticity of substitution,
	etrae(f)	Elasticity of transformation;

$gdxin '.\default.gdx'
$load esubd esubva esubm etrae
$gdxin
* display esubd, esubva, esubm, etrae;

*	----------------------------------------------------------------------
*	Other adjustments:
display "com: Other adjustments:";

*	Determine which factors are sector-specific 
mf(f) = yes$(etrae(f)=0);
sf(f) = yes$(etrae(f)<0);
* display mf,sf;

*	Convert sign of etrae() so that it is non-negative:
etrae(f) = -etrae(f);

*	Define a numeraire region for denominating international
*	transfers:

rnum(r) = yes$(vpm(r)=smax(s,vpm(s)));
* display rnum;

$ontext
*	This code exports data for a pivot-table report:

parameter	rtp(src,i,r)	Taxes on private consumption
		rtg(src,i,r)	Taxes on public consumption
		rtfirm(src,i,j,r)	Taxes on Firms;

rtp("domestic",i,r) = rtpd(i,r);
rtp("imported",i,r) = rtpi(i,r);
rtg("domestic",i,r) = rtgd(i,r);
rtg("imported",i,r) = rtgi(i,r);
rtfirm("domestic",i,j,r) = rtfd(i,j,r);
rtfirm("imported",i,j,r) = rtfi(i,j,r);
$setglobal workbook %ds%_rates
$batinclude pivotdata rtf f i r
$batinclude pivotdata rtp src i r
$batinclude pivotdata rtg src i r
$batinclude pivotdata rtfirm src i j r
$batinclude pivotdata rtxs i r s
$batinclude pivotdata rtms i r s

*	Generate pivot-table data for examining the domestic
*	value share of demand:

parameter	dshare(i,*,r)	Domestic value share;
dshare(i,"c",r)$(vdpm(i,r)*(1+rtpd(i,r))+vipm(i,r)*(1+rtpi0(i,r)))
	= vdpm(i,r)*(1+rtpd(i,r))
		/(vdpm(i,r)*(1+rtpd(i,r))+vipm(i,r)*(1+rtpi0(i,r)));
dshare(i,"g",r)$(vdgm(i,r)*(1+rtgd(i,r))+vigm(i,r)*(1+rtgi0(i,r)))
	= vdgm(i,r)*(1+rtgd(i,r))
		/(vdgm(i,r)*(1+rtgd(i,r))+vigm(i,r)*(1+rtgi0(i,r)));
dshare(i,j,r)$(vdfm(i,j,r)*(1+rtfd0(i,j,r)) + vifm(i,j,r)*(1+rtfi0(i,j,r)))
	= vdfm(i,j,r)*(1+rtfd0(i,j,r))
		/ (vdfm(i,j,r)*(1+rtfd0(i,j,r)) + vifm(i,j,r)*(1+rtfi0(i,j,r)));
set d(*); d(i) = yes; d("c")=yes; d("g")=yes;
$setglobal workbook %ds%dshare
$batinclude pivotdata dshare i d r
$offtext

