$title	The effects of trade liberalization under different market structure.
display "com: The effects of trade liberalization under different market structure.";
$ontext
Copyright (C)   2006 Shiro Takeda
Time-stamp: 	<2008-03-23 18:33:39 Shiro Takeda>
$Id: trade_lib.gms,v 1.1.1.1 2006/11/13 05:57:15 st Exp $
Author: 	Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2004/10/07>
Commentary:

[Acknowledgment]

In writing this program, I have greatly benefited from programs of the Uruguay
round model created by Glenn W. Harrison, Thomas. F. Rutherford, David G. Tarr
and from GTAP6inGAMS package by Thomas. F. Rutherford.  I would like to
express acknowledgment to them.


[Market structure]

PC	Perfect competition model
CD	Cournot model
LGMC	Large group monopolistic competition model (LGMC)
CH	Cournot model with homogeneous varieties
CF	Cournot model with fixed number of firms
QCV	Quantity competition model with non-Cournot conjecutual variation
IC	Integrated market Cournot model
BD	Bertrand competition model

IB	Integrated market Bertrand model
CM	Contestable market model

$offtext

*	Model choice:
$if not setglobal mod_pc $setglobal mod_pc 1
$if not setglobal mod_cd $setglobal mod_cd 1
$if not setglobal mod_lgmc $setglobal mod_lgmc 1
$if not setglobal mod_ch $setglobal mod_ch 1
$if not setglobal mod_cf $setglobal mod_cf 1
$if not setglobal mod_qcv $setglobal mod_qcv 1
$if not setglobal mod_bd $setglobal mod_bd 1
$if not setglobal mod_ic $setglobal mod_ic 1
$if not setglobal mod_ib $setglobal mod_ib 1
$if not setglobal mod_cm $setglobal mod_cm 0

*	Liberalization participants:
$if not setglobal lib_p $setglobal lib_p scn_g

*	Non-zero means the model where CDR is calibrated.
$if not setglobal fl_cdr $setglobal fl_cdr 0

*	Calibraion only
$if not setglobal calib_only $setglobal calib_only 0

*	Benchmark replication only
$if not setglobal br_only $setglobal br_only 0

*	Cleanup calculation only
$if not setglobal cc_only $setglobal cc_only 0

*	Scale check
$if not setglobal sc_chk $setglobal sc_chk 0

*	Flag for AFF.  If non-zero, AFF is assumed to be an imperfectly
*	competitive sector.
$if not setglobal fl_aff $setglobal fl_aff 0

*	Scale factor for EOS (default is unity):
$if not setglobal sc_eos $setglobal sc_eos 1

*	Value of CDR.
$if not setglobal cdr $setglobal cdr 0.15

*	The benchmark number of firms (valid only in model QCV and CDR
*	calibration model).
$if not setglobal n_0 $setglobal n_0 50

*	Tariff on service trade.  If not zero, tariffs on service trade are
*	introduced.
$if not setglobal fl_ser $setglobal fl_ser 1

*	Solver for solving the model (non-zero means MCP, otherwise NLP)
$if not setglobal fl_mcp $setglobal fl_mcp 1

*	Solver for calibration (non-zero means MCP, otherwise NLP)
$if not setglobal fl_mcp_c $setglobal fl_mcp_c 1

*	Alternative model for model IB.
$if not setglobal fl_ib_alt $setglobal fl_ib_alt 0

*	Constant beta:
$if not setglobal fl_beta $setglobal fl_beta 0

*	Value of eod:
$if not setglobal v_eod $setglobal v_eod 0.5

$if not setglobal cdr_file $setglobal cdr_file .\benchmark_cdr_default.gms

$if not setglobal elas_file $setglobal elas_file .\elasticity_default.par

$if not setglobal num_itr $setglobal num_itr 4
$if not setglobal target_rate $setglobal target_rate 20

*	Solver choice for NLP problem:
* option nlp = minos;
option nlp = pathnlp;
* option nlp = conopt;

*	Decimal number.
option decimals = 6;

*	Include gams-f library
$sysinclude gams-f

*	Model list:
set ms_all	Model list	/ PC, CD, LGMC, CH, CF, QCV, BD, IC, CM, IB /
    ms(ms_all)	Type of market structure
    /
$if not %mod_pc%==0	PC         Perfect competition
$if not %mod_cd%==0	CD         Cournot model
$if not %mod_lgmc%==0	LGMC       Large group monopolistic competition
$if not %mod_ch%==0	CH         Cournot model with homogeneous varieties
$if not %mod_cf%==0	CF         Cournot model with fixed number of firms
$if not %mod_qcv%==0	QCV        Quantity competition model with non-Cournot conjecutual variation
$if not %mod_bd%==0	BD         Bertrand model
$if not %mod_ic%==0	IC         Integrated market Cournot model
$if not %mod_ib%==0	IB         Integrated market Bertrand model
$if not %mod_cm%==0	CM         Contestable market model
/
    c_ms(ms_all)	Current market structure
;
parameter
    fl_q	Flag for Quantity competition		/ 0 /
    fl_p	Flag for Price competition		/ 0 /
    fl_lgmc	Flag for LGMC				/ 0 /
    fl_cm	Flag for contestable market		/ 0 /
    fl_h	Flag for homogeneous goods		/ 0 /
    fl_fix	Flag for fixed number of firms		/ 0 /
    fl_int	Flag for integrated market		/ 0 /
    fl_qcv	Flag for non-zero conjecutual variation	/ 0 /
    fl_cdr	/ %fl_cdr% /
    fl_ser	/ %fl_ser% /
    fl_aff	/ %fl_aff% /
    sc_eos	/ %sc_eos% /
    fl_mcp	Solver choice for the model		/ %fl_mcp% /
    fl_mcp_c	Solver choice for calibration		/ %fl_mcp_c% /
    calib_only	Non-zero means calibration only 	/ %calib_only% /
    cc_only	Non-zero means cleanup calculation only	/ %cc_only% /
    br_only	Non-zero means benchmark replication only / %br_only% /
    fl_ib_alt	/ %fl_ib_alt% /
;
display fl_mcp, fl_mcp_c, fl_ib_alt;

*	----------------------------------------------------------------------
*	Import data:
display "com: Import data";
$include ..\trade_lib_data.gms

*	----------------------------------------------------------------------
*	Parameter:
display "com: Parameter";
$include %elas_file%

*	----------------------------------------------------------------------
*	Model definitions
display "com: Model definitions";
$include ..\trade_lib_model.gms

*	----------------------------------------------------------------------
*	Parameters for checking results:
display "com: Parameters for checking results:";
$include ..\trade_lib_par_for_results.gms

*	----------------------------------------------------------------------
*	Model choice
display "com: Model choice";
set
    lp(r)	Liberalization participants /
$include .\%lib_p%.gms
/
    t_map(r,s)
;
alias (lp,lpp);
t_map(lp,lpp) = yes;
parameter
    t_scale	Scale factor for trade taxes;
t_scale = 0;

file titlemaker /'titlemaker.cmd'/;

*	Set temporary values:
option pc_u:3;
pc_u(r,ms) = 0;
pc_scale(ms,k,r) = 0;
pc_n(ms,k,r) = 0;
pc_so(ms,k,r) = 0;
chk_walrus(ms) = 0;

*	For model CM.  When model CM is solved, the values of EOS are changed.
if(%mod_cm%,
    display "com: Contestable market model";
    sig_m(i,r) = sig_a0(i,r);
    sig_dd(i,r) = sig_a0(i,r);
    sig_ff(i,r) = sig_a0(i,r);
    display "Elasticity parameters for contestable market model.";
    display sig_a, sig_m, sig_dd, sig_ff;
);

set	itr	/ 1*%num_itr% /
;
display itr;

*	Loop starts here.
loop(ms,
    c_ms(ms_all) = no;
    c_ms(ms) = yes;
    display c_ms;

$include ..\trade_lib_model_choice.gms

    putclose titlemaker
       '@title Scenario = %lib_p%, model type = ', ms.tl:0;
    execute 'titlemaker.cmd';

*	----------------------------------------------------------------------
*	Execute calibration.
    display "com: Execute calibration";
    if((not calib_only), option solprint = off);
$include  ..\trade_lib_calibration.gms
    if((not calib_only), option solprint = on);
    display fl_int, fl_q, fl_p;
$if not %calib_only%==0 $goto end

*	----------------------------------------------------------------------
*	Import lower bound and initial value
    display "com: Import lower bound and initial value";
$include ..\trade_lib_initial_values.gms

*	----------------------------------------------------------------------
*	Benchmark replication.
    display "com: Benchmark replication.";
    if((not br_only), option solprint = off);
    if(fl_mcp,

	gtap_trade_lib_mcp.workspace = 500;
	gtap_trade_lib_mcp.reslim = 2000;
	gtap_trade_lib_mcp.iterlim = 0;
	solve gtap_trade_lib_mcp using mcp;
	else
	gtap_trade_lib_nlp.workspace = 500;
	gtap_trade_lib_nlp.reslim = 2000;
	gtap_trade_lib_nlp.iterlim = 0;
	solve gtap_trade_lib_nlp using nlp minimizing dum_obj;
    );
    if((not br_only), option solprint = on);

*	Numeraire:
    inc_ra.fx(rnum) = inc_ra.l(rnum);

$if not %br_only%==0 $goto end

*	----------------------------------------------------------------------
*	Cleanup calculation.
    display "com: Cleanup calculation.";
    if((not cc_only), option solprint = off);
    if(fl_mcp,
	gtap_trade_lib_mcp.iterlim = 5000;
	solve gtap_trade_lib_mcp using mcp;
	else
	gtap_trade_lib_nlp.iterlim = 5000;
	solve gtap_trade_lib_nlp using nlp minimizing dum_obj;
    );
    if((not cc_only), option solprint = on);
    display "benchmark tolerance:", gtap_trade_lib_mcp.objval;


$if not %cc_only%==0 $goto end

$if %sc_chk%==0 $goto cf_lib
*	----------------------------------------------------------------------
*	Scale check:
    display "com: Scale check:";

    scale_f(f,r) = 1 + %sc_chk%/100;
    scale_bop(r) = 1 + %sc_chk%/100;
    scale_i(r) = 1 + %sc_chk%/100;

    if(fl_mcp,
	gtap_trade_lib_mcp.iterlim = 5000;
	solve gtap_trade_lib_mcp using mcp;
	else
	gtap_trade_lib_nlp.iterlim = 5000;
	solve gtap_trade_lib_nlp using nlp minimizing dum_obj;
    );

    scale_f(f,r) = 1;
    scale_bop(r) = 1;
    scale_i(r) = 1;

*	----------------------------------------------------------------------
*	Results:
$include ..\trade_lib_output_results.gms

$if not %sc_chk%==0 $goto end

$label cf_lib
*	----------------------------------------------------------------------
*	Liberalization:
    display "com: Liberalization:";

    loop(itr,

	t_scale = 1 - ord(itr) * (1 - %target_rate%/100) / %num_itr%;
	display t_scale;
$include .\lib.shk

	option solprint = off;
	if((ord(itr) eq card(itr)), option solprint = on);
	if(fl_mcp,
	    solve gtap_trade_lib_mcp using mcp;
	else
	    solve gtap_trade_lib_nlp using nlp minimizing dum_obj;
	);

    );

*	----------------------------------------------------------------------
*	Results:
$include ..\trade_lib_output_results.gms

*	----------------------------------------------------------------------
$label end
*	Reset c_ms.
    c_ms(ms_all) = no;

    rtms(i,r,s) = rtms0(i,r,s);
    rtxs(i,r,s) = rtxs0(i,r,s);

*     display pc_u;
*     display chk_walrus;

);
display pc_u;
display chk_walrus;

*	Post processor:
$if exist .\post_proc_default.gms $include .\post_proc_default.gms

display "com: --- Program ends here ---";


* --------------------
* Local Variables:
* mode: gams
* fill-column: 78
* tab-width: 8
* End:

