$title	Imperfectly competitive model
$ontext
Copyright (C)   2006 Shiro Takeda
Time-stamp: 	<2006-07-29 21:42:03 Shiro Takeda>
$Id: trade_lib_calibration.gms,v 1.2 2006/07/29 12:42:58 st Exp $
Author: 	Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2004/10/07>
Commentary:

$offtext
*	----------------------------------------------------------------------
*	Calibration of parameters
display "com: Calibration of parameters";

*	Initial values:
mud0_.l(r) = 0.1;
mux0_.l(r,s) = 0.1;
muxx0_.l(r,s) = 0.1;
mu0_.l(r) = 0.1;

n0_.lo(r) = -inf;
n0_.up(r) = +inf;
vtfc0_.lo(r) = -inf;
vtfc0_.up(r) = +inf;
phi0_.lo(r,s) = -inf;
phi0_.up(r,s) = +inf;
h_qtv0_.l(r) = - 10;

if(fl_qcv,
    phi0_.l(r,s) = 0;
    n0_.fx(r) = %n_0%;
    else
    n0_.l(r) = 10;
    phi0_.fx(r,s) = 0;
);

*	In CDR calibration model, the number of firms is exogenously fixed.
if(fl_cdr,
    n0_.fx(r) = %n_0%;
);

*	----------------------------------------------------------------------
*	Calibration for models except for model LGMC and CM
if((fl_q or fl_p),

    loop(i$smax(r, cdr(i,r)),

	if((fl_cdr and (not fl_qcv)),
	    vtfc0_.l(r) = vtfc_(i,r);
	    else
	    vtfc0_.fx(r) = vtfc_(i,r);
	);

	xx_(r,s) = xx(i,r,s);
	vtqd0_(r) = vtqd0(i,r);
	vtqx0_(r,s) = vtqx0(i,r,s);
	beta0_(r,s) = beta0(i,r,s);
	sh_m_(r,s) = sh_m(i,r,s);
	sh_am_(r) = sh_am(i,r);
	sh_ad_(r) = sh_ad(i,r);
	rto0_(r) = rto(i,r);
	sdd_(r) = sig_dd(i,r);
	sff_(r) = sig_ff(i,r);
	sm_(r) = sig_m(i,r);
	sa_(r) = sig_a(i,r);
	sddi_(r) = 1/sdd_(r);
	sffi_(r) = 1/sff_(r);
	smi_(r) = 1/sm_(r);
	sai_(r) = 1/sa_(r);
	eod0_(r) = eod0(i,r);
	a0_(r) = a0(i,r);

	vtq0_(r) = vtq0(i,r);
	sh_sd_(r) = sh_sd(i,r);
	sh_sx_(r,s) = sh_sx(i,r,s);

	sh_mk_(r,s) = sh_mk(i,r,s);
	rmax(rr,r) = yes$(sh_mk_(rr,r) eq smax(s, sh_mk_(s,r)));
* 	display rmax;

	if(fl_qcv,
*	Calibration for model qcv:
	    calib_par_qcv.iterlim = 9000;
	    solve calib_par_qcv using nlp minimizing obj;
	    else
*	Calibration for other models:
	    if(fl_mcp_c,
*	Calibration by MCP solver:
		calib_par_mcp.iterlim = 9000;
		solve calib_par_mcp using mcp;
		else
*	Calibration by NLP solver:
		calib_par_nlp.iterlim = 9000;
		solve calib_par_nlp using nlp minimizing dum_obj_;
	    );
	);

*	Check calibration results:
	if(fl_qcv,
	    abort$(calib_par_qcv.solvestat <> 1)
		 "Calibration failed. Model not normally completed.",
		 calib_par_qcv.modelstat;
	    abort$(calib_par_qcv.modelstat <> 2)
		 "Calibration failed.  No local optimum found",
		 calib_par_qcv.solvestat;
	    else
	    if(fl_mcp_c,
		abort$(calib_par_mcp.solvestat <> 1)
		     "Calibration failed. Model not normally completed.",
		     calib_par_mcp.modelstat;
		abort$(calib_par_mcp.modelstat <> 1)
		     "Calibration failed.  ", calib_par_mcp.solvestat;
		else
		abort$(calib_par_nlp.solvestat <> 1)
		     "Calibration failed. Model not normally completed.",
		     calib_par_nlp.modelstat;
		abort$(calib_par_nlp.modelstat <> 2)
		     "Calibration failed.  No local optimum found",
		     calib_par_nlp.solvestat;
	    );
	);

*	Results:
	mud0(i,r) = mud0_.l(r);
	mux0(i,r,s) = mux0_.l(r,s);
	muxx0(i,r,s) = muxx0_.l(r,s);
	mu0(i,r) = mu0_.l(r);
	vtfc0(i,r) = vtfc0_.l(r);
	phi0(i,r,s) = phi0_.l(r,s);
	n0(i,r) = n0_.l(r);
	cdr0(i,r) = vtfc0_.l(r)/((1-rto(i,r))*vom(i,r));
 	bphi0(i,r,s)$n0(i,r)
 	    = (1 + (n0(i,r) - 1)*phi0(i,r,s))/n0(i,r);

	eodd0(i,r)$mud0(i,r) = 1/mud0(i,r);
	eodx0(i,r,s)$mux0(i,r,s) = 1/mux0(i,r,s);
	eodc0(i,r)$mu0(i,r) = 1/mu0(i,r);

*	Parameter for integrated Cournot model:
	if((fl_q and fl_int),
	    h_qtv0(i,r) = h_qtv0_.l(r);
	    h_p0(r,i,t) = h_p0_.l(r,t);
	    h_qdv0(i,r) = h_qdv0_.l(r);
	    h_qxv0(i,r,s) = h_qxv0_.l(r,s);
	    h_qd0(r,i,ss) = h_qd0_.l(r,ss);
	    h_qx0(r,i,t,s) = h_qx0_.l(r,t,s);
	    h_qx0(r,i,t,s) = h_qx0_.l(r,t,s);
	    h_pad0(r,i,s) =
		((1 + (n0_.l(r) - 1) * h_p0_.l(r,r))/n0_.l(r))$r_s(r,s)
		+
		(h_p0_.l(r,s))$(not r_s(r,s));
	    h_pm0(r,i,t,s) =
		((beta0_(r,s)
		+ (n0_.l(r) - 1) * beta0_(r,s) * h_p0_.l(r,r))/n0_.l(r))$r_s(r,t)
		+
		(beta0_(t,s) * h_p0_.l(r,t))$(not r_s(r,t));
	    h_pam0(r,i,s) =
		sum(tt$sh_m_(tt,s), sh_m_(tt,s) * h_pm0(r,i,tt,s));
	    h_pa0(r,i,s) =
		(sh_ad_(s) * h_pad0(r,i,s))$sh_ad_(s)
		+ (sh_am_(s) * h_pam0(r,i,s))$sh_am_(s)

	);

*	Check zero profit condition:
	profitchk(i,r)$vtfc0(i,r)
	    = ((1-rto(i,r))
	    * (mud0(i,r)*vtqd0(i,r)
	    + sum(s$vtqx0(i,r,s), mux0(i,r,s)*vtqx0(i,r,s)))
	    - vtfc0(i,r))$(not fl_int)
	    +
	    ((1-rto(i,r)) * mu0(i,r) *vtq0(i,r) - vtfc0(i,r))$fl_int;
	profitchk(i,r) = round(profitchk(i,r), 5);

    );

*     Check results:
    abort$(smin((i,r), mud0(i,r)) lt 0)
	 "Calibrated markup rate is negative!!!!", mud0;
    abort$(smin((i,r,s), mux0(i,r,s)) lt 0)
	 "Calibrated markup rate is negative!!!!", mux0;
    abort$(smin((i,r), mu0(i,r)) lt 0)
	 "Calibrated markup rate is negative!!!!", mu0;
    abort$(smin((i,r), n0(i,r)) lt 0)
	 "The number of firms is negative!!!!", n0;
    abort$(smin((i,r), vtfc0(i,r)) lt 0)
	 "Fixed cost is negative!!!!", vtfc0;
    abort$(smax((i,r), cdr0(i,r)) gt 1)
	 "CDR exceeds one!!!!", cdr0;
    abort$(smax((i,r), abs(profitchk(i,r))) gt 1.e-4)
	 "Profit is not zero!!!!", profitchk;

);

*	----------------------------------------------------------------------
*	Calibration for model LGMC
if(fl_lgmc,

    loop(i$smax(r, cdr(i,r)),

*	In LGMC, the benchmark number of firms is normalized to one.  This
*	normalization does not affect results (that is, rates of changes in
*	variables do not depend on this normalization).
	n0(i,r) = 1;

*	Markup rates are equal to the inverse of EOS:
	mud0(i,r)$vtqd0(i,r) = 1 / sig_dd(i,r);
	muxx0(i,r,s)$vtqx0(i,r,s) = 1 / sig_ff(i,r);
	mux0(i,r,s)$vtqx0(i,r,s) = muxx0(i,r,s) / beta0(i,r,s);

*	Conjectural variations are irrelevant.  So, they are set to zero.
	phi0(i,r,s) = 0;
	bphi0(i,r,s) = 0;

*	In LGMC, value of fixed cost is calibrated so that zero profit
*	condition is satisfied.
	vtfc0(i,r) =
	    (1-rto(i,r))
	    * (mud0(i,r)*vtqd0(i,r) + sum(s$vtqx0(i,r,s),
	    mux0(i,r,s)*vtqx0(i,r,s)));
    );
);

*	----------------------------------------------------------------------
*	Calibration for model MC (contestable market model)
if(fl_cm,

    loop(i$smax(r, cdr(i,r)),

*	In contestable market model, the benchmark number of firms is
*	normalized to one.  This normalization does not affect results (that
*	is, rates of changes in variables do not depend on this
*	normalization).
	n0(i,r) = 1;

*	Markup rates are irrelevant.  So, they are set to zero.
	mud0(i,r)$vtqd0(i,r) = 0;
	muxx0(i,r,s)$vtqx0(i,r,s) = 0;
	mux0(i,r,s)$vtqx0(i,r,s) = 0;

*	Value of total fixed cost:
	vtfc0(i,r) = vtfc_(i,r);

    );
);

*	----------------------------------------------------------------------
*	Display calibrated parameters:
display "com: Display calibrated parameters:";
display mud0, mux0, n0;
* display muxx0;
display phi0, bphi0;
display vtfc0;
display cdr0;
if(fl_int, display mu0;);
if(c_ms("ic"),
    display h_qtv0, h_p0, h_qdv0, h_qxv0, h_qd0, h_qx0, h_pad0,
	    h_pm0, h_pam0, h_pa0;
);

*	----------------------------------------------------------------------
*	Assign values to parameters:
    display "com: Assign values to parameters:";

*	----------------------------------------------------------------------
*	Models except for model CM:
if((not fl_cm),

*	Unit cost is normalized to one.  This normalization does not affect
*	results.
    cy0(i,r) = 1;

    if(fl_int,
	py0(k,r) = cy0(k,r)/((1-rto(k,r))*(1-mu0(k,r)));
	pd0(k,r) = py0(k,r);
	px0(k,r,s) = py0(k,r);
	else
	pd0(k,r) = cy0(k,r)/((1-rto(k,r))*(1-mud0(k,r)));
	px0(k,r,s) = cy0(k,r)/((1-rto(k,r))*(1-mux0(k,r,s)));
    );
*     display pd0, px0, n0;

    qd0(k,r) = vtqd0(k,r)/(pd0(k,r)*n0(k,r));
    qx0(k,r,s) = vtqx0(k,r,s)/(px0(k,r,s)*n0(k,r));
*     display qd0, qx0;

    vfc0(k,r) = vtfc0(k,r) / n0(k,r);
    tfc0(k,r) = vtfc0(k,r) / cy0(k,r);
    fc0(k,r) = tfc0(k,r) / n0(k,r);
*     display vtfc0, fc0;

    qtt0(k,r)
	= (qd0(k,r) + sum(s, qx0(k,r,s))) + fc0(k,r);
*     display qtt0;
    qt0(k,r)
	= qd0(k,r) + sum(s, qx0(k,r,s));
*     display qt0;
    ac0(k,r)$qt0(k,r) = cy0(k,r) * qtt0(k,r) / qt0(k,r);
*     display ac0;

);

*	----------------------------------------------------------------------
*	Model CM (contestable market model)
if(fl_cm,

*	Average cost is normalized to one:
    ac0(k,r) = 1;

    pd0(k,r) = ac0(k,r)/(1-rto(k,r));
    px0(k,r,s) = ac0(k,r)/(1-rto(k,r));
    py0(k,r) = ac0(k,r)/(1-rto(k,r));
*     display pd0, px0, py0;

    qd0(k,r) = vtqd0(k,r)/(pd0(k,r)*n0(k,r));
    qx0(k,r,s) = vtqx0(k,r,s)/(px0(k,r,s)*n0(k,r));
*     display qd0, qx0;

    qt0(k,r) = qd0(k,r) + sum(s, qx0(k,r,s));
*     display qt0;

    vfc0(k,r) = vtfc0(k,r) / n0(k,r);
*     display vtfc0, vfc0;

    cy0(k,r)$qt0(k,r)
	= (ac0(k,r) * qt0(k,r) - vfc0(k,r)) / qt0(k,r);
*     display cy0;

    tfc0(k,r)$cy0(k,r) = vtfc0(k,r) / cy0(k,r);
    fc0(k,r)$n0(k,r) = tfc0(k,r) / n0(k,r);
*     display vtfc0, fc0;

    qtt0(k,r) = (qd0(k,r) + sum(s, qx0(k,r,s))) + fc0(k,r);
*     display qtt0;

*	Irrelevant values:
    mud0(i,r) = 0;
    mux0(i,r,s) = 0;
    muxx0(i,r,s) = 0;
*     display "Irrelevant values: ", mud_0, mux_0, muxx_0;
);

* --------------------
* Local Variables:
* mode: gams
* fill-column: 78
* End:
