$ontext
Copyright (C)   2006 Shiro Takeda
Time-stamp: 	<2006-07-29 21:41:36 Shiro Takeda>
$Id: trade_lib_model.gms,v 1.2 2006/07/29 12:42:58 st Exp $
version:
Author: 	Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2005/04/02>
$offtext

*	----------------------------------------------------------------------
*	Parameters for calibration:
parameter
    beta0(i,r,s)	Value share of goods in leontief aggregate
    qx0(i,r,s)		Export supply of each firm
    qd0(i,r)		Domestic supply of each firm
    vtqx0(i,r,s)	Total value of export supply
    vtqd0(i,r)		Total value of domestic supply
    vtq0(i,r)		Total value of supply
    vtfc_(i,r)	 	Benchmark total fixed cost
    xx(i,r,s)		Value of supply
    sh_mk(i,*,s)	Market share
    eod0(i,r)		Elasticity of demand for Armington goods
;
parameter
    beta0_(r,s)		Value share of goods in leontief aggregate
    vtqx0_(r,s)		Total value of export supply
    vtqd0_(r)		Total value of domestic supply
    vtq0_(r)		Total value of supply
    rto0_(r)		Output tax rate
    a0_(r)		Armington demand
    sa_(r)		Armington elasticity,
    sm_(r)		import EOS
    sdd_(r)		dd EOS
    sff_(r)		ff EOS
    sai_(r)		Inverse of Armington elasticity,
    smi_(r)		Inverse of import EOS
    sddi_(r)		Inverse of dd EOS
    sffi_(r)		Inverse of ff EOS
    sh_m_(r,s)		Share of import from each region
    sh_am_(r)		Share of aggregate import
    sh_ad_(r)		Share of domestic goods
    xx_(r,s)		Value of supply
    sh_mk_(r,s)		Market share
    eod0_(r)		Elasticity of demand for Armington goods
    sh_sd_(r)		Domestic supply share in total supply
    sh_sx_(r,s)		Export supply share in total supply
;
*	Value share of exports (imports):
beta0(i,s,r)$vxmd(i,s,r)
    = pmm0(i,s,r)*vxmd(i,s,r) / (pmm0(i,s,r)*vxmd(i,s,r) + vtwr_(i,s,r));

*	Value of total fixed cost:
vtfc_(i,r) = cdr(i,r) * (1-rto(i,r)) * (1-sh_trns(i,r)) * vom(i,r);

*	Value of total exports at FOB price:
vtqx0(i,s,r) = vxmd(i,s,r);

*	Value of total domestic supply:
vtqd0(i,r) = vdm(i,r);

vtq0(i,r) = vtqd0(i,r) + sum(s, vtqx0(i,r,s));

xx(i,s,r) = vxmd(i,s,r);
xx(i,s,s) = vdm(i,s);

sh_mk(i,s,r)$vxmd(i,s,r) =
    (vxmd(i,s,r) * pmx0(i,s,r) + vtwr_(i,s,r) * pmt0(i,s,r) )
    / (vim(i,r) + vdm(i,r));
sh_mk(i,r,r)$vdm(i,r) =
    vdm(i,r) / (vim(i,r) + vdm(i,r));
sh_mk(i,"total",r) = sum(s, sh_mk(i,s,r));

*	----------------------------------------------------------------------
*	Price elasticity of demand:

*	To be precise, price elasticity of Armington demand (eod0(i,r)) is
*	determined by the following relation.

*	eod0(i,r)$a0(i,r) = vcm(i,r) / a0(i,r);

*	However, for some goods (e.g. MIN), eod0(i,r) is equal to zero because
*	all Armington demand are intermediate demand.  If eod0 = 0, some
*	markup rates cannot be defined.  Here, we assume eod0(i,r) = 0.5.
eod0(i,r) = 0.5;

set  rmax(r,s)		Mapping set;
$ontext
rmax(r,s) has the value yes if the share of supply from region r is the
largest in market of region s.
$offtext

*	----------------------------------------------------------------------
*	Calibration:
display "com: Calibration:";

*	Calibrated parameters:
parameter
    mud0(i,r)		Calibrated benchmark markup rate for domestic market
    mux0(i,r,s)		Calibrated benchmark markup rate for exports
    muxx0(i,r,s)	Calibrated benchmark markup rate for exports (inc. transport cost)
    n0(i,r)		Calibrated benchmark number of firms
    vtfc0(i,r)		Calibrated benchmark total fixed cost
    vfc0(i,r)		Value of fixed cost of each firm
    tfc0(i,r)		Total fixed cost
    fc0(i,r)		Fixed cost of each firm
    pd0(i,r)		Reference price of domestic supply
    px0(i,r,s)		Reference price of export supply
    qtt0(i,r)		Total input (variable input + fixed input)
    qt0(i,r)		Total ouput
    cy0(i,r)		Calibrated benchmark unit cost

    tep0(i,r)		"(Excess) profit"
    ep0(i,r)		"(Excess) profit"

    phi0(i,r,s)		"Calibrated conjectural variation (delta x_j/ delta x_i)"
    bphi0(i,r,s)	"Calibrated conjectural variation (delta ln X/ delta ln x_i)"

*	Parameters for integrated market model:
    py0(i,r)		Reference price for ouput
    mu0(i,r)		Calibrated benchmark markup rate

*	Parameters for integrated Bertrand model:
    eodd0(i,r)		Elasticity of domestic demand
    eodx0(i,r,s)	Elasticity of export demand
    eodc0(i,r)		Elasticity of total demand

*	Parameter for integrated Cournot model:
    h_qtv0(i,r)		Rate of change in own total supply
    h_p0(r,i,t)		Conjectured rate of change in price of rival firm
    h_qdv0(i,r)		Conjectured ROC in own domestic supply
    h_qxv0(i,r,s)	Conjectured ROC in own export supply
    h_qd0(r,i,ss)	Conjectured ROC in domestic supply of rival firms
    h_qx0(r,i,t,s)	Conjectured ROC in export supply of rival firms:
    h_pad0(r,i,ss)	Conjectured ROC in p_ad:
    h_pm0(r,i,t,s)	Conjectured ROC in p_m:
    h_pam0(r,i,s)	Conjectured ROC in p_am:
    h_pa0(r,i,s)	Conjectured ROC in p_a:

*	Parameter for CDR calibration model:
    cdr0(i,r)		Calibrated CDR

*	Parameter for contestable market model:
    ac0(i,r)		Calibrated benchmark average cost (for contestable market model)
;
*	Temporary value:
cy0(i,r) = 0;
ac0(i,r) = 0;
parameter
    profitchk	 	Profit check (this must be zero);

*	Variables for calibration:
variables
    mud0_(r)		Benchmark markup rate for domestic market
    mux0_(r,s)		Benchmark markup rate for exports
    muxx0_(r,s)		Benchmark markup rate for exports (inc. transport cost)

    n0_(r)		Benchmark number of firms

*	Variables for conjectural variation model:
    phi0_(r,s)		Conjecutual variation parameter
    obj			Objective variable

*	Variables for integrated market model:
    mu0_(r)		Benchmark markup rate

*	Variables for integrated market Cournot model:
    h_qtv0_(r)		Rate of change in own total supply
    h_p0_(r,t)		Conjectured rate of change in price of rival firm
    h_qdv0_(r)		Conjectured ROC in own domestic supply
    h_qxv0_(r,s)	Conjectured ROC in own export supply
    h_qd0_(r,ss)	Conjectured ROC in domestic supply of rival firms
    h_qx0_(r,t,s)	Conjectured ROC in export supply of rival firms:
    h_pad0_(r,ss)	Conjectured ROC in p_ad:
    h_pm0_(r,t,s)	Conjectured ROC in p_m:
    h_pam0_(r,s)	Conjectured ROC in p_am:
    h_pa0_(r,s)		Conjectured ROC in p_a:

*	Variables for CDR calibration model:
    vtfc0_(r)		Benchmark total fixed cost

*	Equations for NLP model
    dum_obj_		Dummy objective variable
;
*	Equations for calibration:
equations
    e_n0_(r)		Zero profit condition
    e_mud0_(r)		Benchmark markup rate for domestic market
    e_mux0_(r,s)	Benchmark markup rate for exports
    e_muxx0_(r,s)	Benchmark markup rate for exports (adjusted by transport cost)

*	Equations for conjectural variation model:
    e_obj		Objective variable

*	Equations for integrated market model
    e_mu0_(r)		Benchmark markup rate

*	Variables for integrated market Cournot model:
    e_h_qtv0_(r)	Rate of change in own total supply
    e_h_p0_(r,t)	Conjectured rate of change in price of rival firm
    e_h_qdv0_(r)	Conjectured ROC in own domestic supply
    e_h_qxv0_(r,s)	Conjectured ROC in own export supply
    e_h_qd0_(r,s)	Conjectured ROC in domestic supply of rival firms
    e_h_qx0_(r,t,s)	Conjectured ROC in export supply of rival firms:
    e_h_pad0_(r,ss)	Conjectured ROC in p_ad:
    e_h_pm0_(r,t,s)	Conjectured ROC in p_m:
    e_h_pam0_(r,s)	Conjectured ROC in p_am:
    e_h_pa0_(r,s)	Conjectured ROC in p_a:

*	Equations for CDR calibration model:
    e_vtfc0_(r)		Zero profit condition

*	Equations for NLP model
    e_dum_obj_		Dummy objective variable
;
*	----------------------------------------------------------------------
*	Equation definitions for calibration:
display "com: Equation definitions for calibration:";

*	Zero profit condition:
e_n0_(r)$vtq0_(r) ..

    vtfc0_(r)/(1-rto0_(r)) =e=
    ((vtqd0_(r) * mud0_(r))$vtqd0_(r)
	+ sum(s$vtqx0_(r,s), vtqx0_(r,s) * mux0_(r,s))
    )$(not fl_int)
    +
    (mu0_(r)*vtq0_(r))$fl_int
    ;
*	Markup rate for domestic supply:
e_mud0_(r)$(sh_ad_(r) and (not c_ms("ic"))) ..

    (mud0_(r)*n0_(r) - sddi_(r)*n0_(r)
	- (sai_(r) - sddi_(r))
	* (1 + (n0_(r) - 1) * phi0_(r,r))
	- (1/eod0_(r) - sai_(r))
	* (sh_ad_(r) + (n0_(r) - sh_ad_(r)) * phi0_(r,r))
    )$(fl_q and (not fl_h))
    +
    (n0_(r) - sdd_(r) * n0_(r) * mud0_(r)
	- mud0_(r) * (sa_(r) - sdd_(r) + sh_ad_(r) * (eod0_(r) - sa_(r)))
    )$fl_p
    +
    (mud0_(r)*n0_(r)
	- sai_(r) * (1 + (n0_(r) - 1) * phi0_(r,r))
	- (1/eod0_(r) - sai_(r))
	* (sh_ad_(r) + (n0_(r) - sh_ad_(r)) * phi0_(r,r))
    )$(fl_q and fl_h)
    =e= 0;

*	Markup rate for export supply:
e_muxx0_(r,s)$(sh_m_(r,s) and (not c_ms("ic"))) ..

    (muxx0_(r,s)*n0_(r)
	- sffi_(s)*n0_(r)
	- (smi_(s) - sffi_(s))
	* (1 + (n0_(r) - 1) * phi0_(r,s))
	- (sai_(s) - smi_(s))
	* (sh_m_(r,s) + (n0_(r) - sh_m_(r,s)) * phi0_(r,s))
	- (1/eod0_(s) - sai_(s))
	* ((sh_am_(s)*sh_m_(r,s))
	    + (n0_(r) - (sh_am_(s)*sh_m_(r,s))) * phi0_(r,s))
    )$(fl_q and (not fl_h))
    +
    (n0_(r) - sff_(s) * n0_(r) * muxx0_(r,s)
	- muxx0_(r,s) * (sm_(s) - sff_(s)
	    + sh_m_(r,s) * (sa_(s) - sm_(s)
		+ sh_am_(s) * (eod0_(s) - sa_(s))))
    )$fl_p
    +
    (muxx0_(r,s)*n0_(r)
	- smi_(s) * (1 + (n0_(r) - 1) * phi0_(r,s))
	- (sai_(s) - smi_(s))
	* (sh_m_(r,s) + (n0_(r) - sh_m_(r,s)) * phi0_(r,s))
	- (1/eod0_(s) - sai_(s))
	* ((sh_am_(s)*sh_m_(r,s))
	    + (n0_(r) - (sh_am_(s)*sh_m_(r,s))) * phi0_(r,s))
    )$(fl_q and fl_h)
    =e= 0;

*	Markup rate for export supply:
e_mux0_(r,s)$(sh_m_(r,s) and (not c_ms("ic"))) ..

    mux0_(r,s) =e= muxx0_(r,s) / beta0_(r,s);

*	Common markup rate for integrated market:
e_mu0_(r)$(fl_int and vtq0_(r)) ..

    ((mu0_(r) - sh_sd_(r) * mud0_(r)
	- sum(s$sh_sx_(r,s), sh_sx_(r,s) * mux0_(r,s)))$(fl_ib_alt)
	+
	(1/mu0_(r) - sh_sd_(r) / mud0_(r)
	    - sum(s$sh_sx_(r,s), sh_sx_(r,s) / mux0_(r,s))
	)$(not fl_ib_alt)
    )$fl_p
    +
    (mu0_(r) * h_qtv0_(r) + 1)$fl_q

    =e= 0;

*	Zero profit condition:
e_vtfc0_(r)$vtq0_(r) ..

    vtfc0_(r)/(1-rto0_(r)) =e=
    ((vtqd0_(r) * mud0_(r))$vtqd0_(r)
	+ sum(s$vtqx0_(r,s), vtqx0_(r,s) * mux0_(r,s))
    )$(not fl_int)
    +
    (mu0_(r)*vtq0_(r))$fl_int
    ;
*	Dummy objective variable for NLP solve:
e_dum_obj_ ..

    dum_obj_ =e= 1;

*	Objective function for model QCV
e_obj ..

    obj =e= sum((r,s,t)$(xx_(s,r)$rmax(t,r)), xx_(s,r) *
	(phi0_(s,r)-phi0_(t,r))*(phi0_(s,r)-phi0_(t,r)));

*	----------------------------------------------------------------------
*	FUNCTION definitions for integrated Cournot model: These functions are
*	valid only in model IC.

*	Conjectured ROC in p_ad:
h_pad0_(r,s) ==
    ((1 + (n0_(r) - 1) * h_p0_(r,r))/n0_(r))$r_s(r,s)
    +
    (h_p0_(r,s))$(not r_s(r,s))
    ;
*	Conjectured ROC in p_m:
h_pm0_(r,t,s) ==
    ((beta0_(r,s)
	+ (n0_(r) - 1) * beta0_(r,s) * h_p0_(r,r))/n0_(r)
    )$r_s(r,t)
    +
    (beta0_(t,s) * h_p0_(r,t))$(not r_s(r,t))
    ;
*	Conjectured ROC in p_am:
h_pam0_(r,s) ==
    sum(tt$sh_m_(tt,s), sh_m_(tt,s) * h_pm0_(r,tt,s))
    ;
*	Conjectured ROC in p_a:
h_pa0_(r,s) ==
    (sh_ad_(s) * h_pad0_(r,s))$sh_ad_(s)
    + (sh_am_(s) * h_pam0_(r,s))$sh_am_(s)
    ;
*	----------------------------------------------------------------------
*	Equations for integrated market Cournot model:

*	ROC in own total supply:
e_h_qtv0_(r)$(c_ms("ic") and vtq0_(r)) ..

    h_qtv0_(r) =e=
    (sh_sd_(r) * h_qdv0_(r))$sh_sd_(r)
    + sum(s$sh_sx_(r,s), sh_sx_(r,s) * h_qxv0_(r,s));

*	Conjectured ROC in prices of rival firms (Cournot conjecture):
e_h_p0_(r,t)$(c_ms("ic") and vtq0_(t)) ..

    (sh_sd_(t) * h_qd0_(r,t))$sh_sd_(t)
    + sum(s$sh_sx_(t,s), sh_sx_(t,s) * h_qx0_(r,t,s))
    =e= 0;

*	Conjectured ROC in own domestic supply
e_h_qdv0_(r)$(c_ms("ic") and sh_sd_(r)) ..

    h_qdv0_(r) =e=
    - sdd_(r)
    + (sdd_(r) - sa_(r)) * h_pad0_(r,r)
    + (sa_(r) - eod0_(r)) * h_pa0_(r,r);

*	Conjectured ROC in own export supply:
e_h_qxv0_(r,s)$(c_ms("ic") and sh_sx_(r,s)) ..

    h_qxv0_(r,s) =e=
    - sff_(s) * beta0_(r,s)
    + (sff_(s) - sm_(s)) * h_pm0_(r,r,s)
    + (sm_(s) - sa_(s)) * h_pam0_(r,s)
    + (sa_(s) - eod0_(s)) * h_pa0_(r,s);

*	Conjectured ROC in domestic supply of rival firms
e_h_qd0_(r,s)$(c_ms("ic") and sh_sd_(s)) ..

    h_qd0_(r,s) =e=
    - sdd_(s) * h_p0_(r,s)
    + (sdd_(s) - sa_(s)) * h_pad0_(r,s)
    + (sa_(s) - eod0_(s)) * h_pa0_(r,s);

*	Conjectured ROC in export supply of rival firms:
e_h_qx0_(r,t,s)$(c_ms("ic") and sh_sx_(t,s)) ..

    h_qx0_(r,t,s) =e=
    - sff_(s) * beta0_(t,s) * h_p0_(r,t)
    + (sff_(s) - sm_(s)) * h_pm0_(r,t,s)
    + (sm_(s) - sa_(s)) * h_pam0_(r,s)
    + (sa_(s) - eod0_(s)) * h_pa0_(r,s);

*	----------------------------------------------------------------------
*	Model declaration for calibration:
display "com: Model declaration for calibration:";

model calib_par_qcv Calibration of parameters for model QCV

      / e_obj, e_n0_, e_mud0_, e_muxx0_, e_mux0_ /;

model calib_par_mcp Calibration of parameters by MCP solver.

      / e_n0_.n0_, e_mud0_.mud0_, e_muxx0_.muxx0_, e_mux0_.mux0_,
	e_vtfc0_.vtfc0_, e_mu0_.mu0_, e_h_qtv0_.h_qtv0_, e_h_p0_.h_p0_,
	e_h_qdv0_.h_qdv0_, e_h_qxv0_.h_qxv0_, e_h_qd0_.h_qd0_, e_h_qx0_.h_qx0_
	/;

model calib_par_nlp Calibration of parameters by NLP solver.

      / e_n0_, e_mud0_, e_muxx0_, e_mux0_, e_vtfc0_, e_mu0_, e_h_qtv0_,
	e_h_p0_, e_h_qdv0_, e_h_qxv0_, e_h_qd0_, e_h_qx0_, e_dum_obj_ /;

*	----------------------------------------------------------------------
*	Variable declaration for model
display "com: Variable declaration for model";

variables
    q_d(i,r)		Domestic supply (firm level)
    q_x(i,r,s)		Export supply (firm level)
    n(i,r)		The number of firms
    q_tt(i,r)		Total input (variable input + fixed input) of each firm
    q_t(i,*)		Total output of each firm
    mu_d(i,r)		Markup rate of domestic supply
    mu_x(i,r,s)		Markup rate of foreign supply
    mu_xx(i,r,s)	Markup rate of foreign supply
    ac(i,r)		Average cost

    y(i,r)		Production
    y_xt(i,r)		Production of inputs for transport sector in K
    a(i,r)		Armington aggregation
    ad(i,r)		Aggregation of domestic varieties
    am(i,r)		Import aggregation
    m(i,r,s)		Aggregation of foreign varieties
    u(r)		Utility
    yt			Transport sector
    p_y(i,r)		Price index of ouputs
    p_d(i,r)		Price of domestic supply
    p_x(i,r,s)		Price of export supply
    p_com(i,r)		Common supply price in the integrated market model
    p_ad(i,r)		Price index of domestic goods
    p_am(i,r)		Price index of import goods
    p_m(i,r,s)		Price of import
    p_t			Price of transport services
    p_a(i,r)		Price index of Armington goods
    p_f(f,r)		Price of primary factors
    p_u(r)		Price index of utility
    inc_ra(r)		Income of representative agent
    ep(i,r)		"(Excess) profit"
    mu(i,r)		Common markup rates
    h_p(r,i,t)		Conjectured rate of change in price of rival firm
    h_pm(r,i,t,s)	Conjectured ROC in p_m:
    h_pam(r,i,s)	Conjectured ROC in p_am:
    h_pad(r,i,s)	Conjectured ROC in p_ad:
    h_pa(r,i,s)		Conjectured ROC in p_a:
    eod_d(i,r)		Elasticity of domestic demand
    eod_x(i,r,s)	Elasticity of export demand
    eod_c(i,r)		Elasticity of total demand

    dum_obj		Dummy objective variable for NLP model
;
*	----------------------------------------------------------------------
*	Equation declaration for model
display "com: Equation declaration for model";
equations
    e_q_d(i,r)		Domestic supply (firm level)
    e_q_x(i,r,s)	Export supply (firm level)
    e_n(i,r)		The number of firms
    e_q_tt(i,r)		Total input (variable input + fixed input) of each firm
    e_q_t(i,*)		Total output of each firm
    e_mu_d(i,r)		Markup rate of domestic supply
    e_mu_x(i,r,s)	Markup rate of foreign supply
    e_mu_xx(i,r,s)	Markup rate of foreign supply
    e_ac(i,r)		Average cost
    e_y(i,r)		Production
    e_y_xt(i,r)		Production of inputs for transport sector in K
    e_a(i,r)		Armington aggregation
    e_ad(i,r)		Aggregation of domestic varieties
    e_am(i,r)		Import aggregation
    e_m(i,r,s)		Aggregation of foreign varieties
    e_u(r)		Utility
    e_yt		Transport sector
    e_p_y(i,r)		Price index of ouputs
    e_p_d(i,r)		Price of domestic supply
    e_p_x(i,r,s)	Price of export supply
    e_p_com(i,r)	Common supply price in the integrated market model
    e_p_ad(i,r)		Price index of domestic goods
    e_p_am(i,r)		Price index of import goods
    e_p_m(i,r,s)	Price of import
    e_p_t		Price of transport services
    e_p_a(i,r)		Price index of Armington goods
    e_p_f(f,r)		Price of primary factors
    e_p_u(r)		Price index of utility
    e_inc_ra(r)		Income of representative agent
    e_ep(i,r)		Excess profit
    e_mu(i,r)		Common markup rates
    e_h_p(r,i,t)	Conjectured rate of change in price of rival firm
    e_h_pm(r,i,t,s)	Conjectured ROC in p_m:
    e_h_pam(r,i,s)	Conjectured ROC in p_am:
    e_eod_d(i,r)	Elasticity of domestic demand
    e_eod_x(i,r,s)	Elasticity of export demand
    e_eod_c(i,r)	Elasticity of total demand

    e_dum_obj		Dummy objective variable for NLP model
;

*	----------------------------------------------------------------------
*	Function definitions:
display "com: Function definitions for model:";

*	----------------------------------------------------------------------
*	Price index and unit cost

*	Price index of composite primary factor:
p_pf(i,r) ==
    (
	((sum(f$sh_f(f,i,r),
	    sh_f(f,i,r)
	    * ((1+rtf(f,i,r)) * p_f(f,r) / pf0(f,i,r))**(1-sig_pf(i,r)))
	)**(1/(1-sig_pf(i,r))))$(sig_pf(i,r) ne 1)
	+
	(prod(f$sh_f(f,i,r),
	    ((1+rtf(f,i,r)) * p_f(f,r) / pf0(f,i,r))**(sh_f(f,i,r)))
	)$(sig_pf(i,r) eq 1)
    )$vad(i,r);

*	Unit cost of production:
c_y(i,r) ==
    (sum(j$sh_i(j,i,r),
	sh_i(j,i,r) * (1 + rti(j,i,r)) * p_a(j,r) / pai0(j,i,r))
	+ (sh_pf(i,r) * p_pf(i,r))$vad(i,r)
    )$vom(i,r);

*	Unit cost of Armington aggregation:
c_a(i,r) ==
    (
	(((sh_am(i,r) * (p_am(i,r))**(1-sig_a(i,r)))$sh_am(i,r)
	+ (sh_ad(i,r) * (p_y(i,r))**(1-sig_a(i,r)))$sh_ad(i,r)
	)**(1/(1-sig_a(i,r))))$l(i)
	+
	(((sh_am(i,r) * (p_am(i,r))**(1-sig_a(i,r)))$sh_am(i,r)
	    + (sh_ad(i,r) * (p_ad(i,r))**(1-sig_a(i,r)))$sh_ad(i,r)
	)**(1/(1-sig_a(i,r))))$k(i)
    )$a0(i,r)
;
*	Unit cost of domestic goods:
c_ad(i,r) ==
    ((n(i,r))**(1/(1-sig_dd(i,r))) * p_d(i,r)
    )$(k(i) and vdm(i,r) and (not fl_h));

*	Unit cost of consumption aggregation:
c_u(r) ==
    (
	(sum(i$sh_c(i,r),
	    sh_c(i,r) * ((1+rtc(i,r))*p_a(i,r) / pc0(i,r))**(1-sig_c))
	)**(1/(1-sig_c))
    )$(sig_c ne 1)
    +
    (
	prod(i$sh_c(i,r),
	    ((1+rtc(i,r))*p_a(i,r) / pc0(i,r))**(sh_c(i,r)))
    )$(sig_c eq 1);

*	Unit cost of import aggregation:
c_am(i,r) ==
    (
	((sum(s$sh_m(i,s,r),
	    sh_m(i,s,r) * ((1+rtms(i,s,r))*p_m(i,s,r)/pmt0(i,s,r))**(1-sig_m(i,r))
	))**(1/(1-sig_m(i,r)))
	)$(sig_m(i,r) ne 1)
	+
	(prod(s$sh_m(i,s,r),
	    ((1+rtms(i,s,r))*p_m(i,s,r)/pmt0(i,s,r))**(sh_m(i,s,r)))
	)$(sig_m(i,r) eq 1)
    )$vim(i,r);

*	Unit cost of import:
c_m(i,r,s) ==
    ((n(i,r))**(1/(1-sig_ff(i,s))) * p_x_(i,r,s)
    )$(k(i) and vxmd(i,r,s) and (not fl_h));

*	Price of export including transport cost (CIF price of export):
p_x_(i,r,s) ==
    (
	(gamma(i,r,s)*(1-rtxs(i,r,s))*p_y(i,r)/pmm0(i,r,s)
	    + ((1-gamma(i,r,s))*(tau(i,r,s)*p_t/tau0(i,r,s)))$tau0(i,r,s)
	)$l(i)
	+
	(gamma(i,r,s)*(1-rtxs(i,r,s))*p_x(i,r,s)/pmm0(i,r,s)
	    + ((1-gamma(i,r,s))*(tau(i,r,s)*p_t/tau0(i,r,s)))$tau0(i,r,s)
	)$k(i)
    )$vxmd(i,r,s);

*	Unit cost of transport services:
c_t ==
    ((sum((i,r)$sh_t(i,r), sh_t(i,r) * (p_y(i,r))**(1-sig_t))
    )**(1/(1-sig_t)))$(sig_t ne 1)
    +
    (prod((i,r)$sh_t(i,r), (p_y(i,r))**(sh_t(i,r))))$(sig_t eq 1);

*	Price of investment goods:
p_inv(r) == sum(cgd, p_y(cgd,r));

*	----------------------------------------------------------------------
*	Demand and supply functions:

*	Demand for primary factors:
a_f(f,i,r) ==
    (vfm(f,i,r) *
	(p_pf(i,r) / ((1+rtf(f,i,r))*p_f(f,r)/pf0(f,i,r)))**(sig_pf(i,r))
    )$vfm(f,i,r);

*	Final demand:
a_c(i,r) ==
    (v_cm(i,r) * (c_u(r) / ((1+rtc(i,r)) * p_a(i,r) / pc0(i,r)))**(sig_c)
    )$v_cm(i,r);

*	Demand for domestic goods:
a_ad(i,r) ==
    (
	(ad0(i,r) * (c_a(i,r)/p_y(i,r))**(sig_a(i,r)))$l(i)
	+
	(ad0(i,r) * (c_a(i,r)/p_ad(i,r))**(sig_a(i,r)))$k(i)
    )$ad0(i,r);

*	Demand for import goods:
a_am(i,r) ==
    (am0(i,r) * (c_a(i,r)/p_am(i,r))**(sig_a(i,r)))$am0(i,r);

*	Demand for import:
a_m(i,r,s) ==
    (m0(i,r,s)
	* (c_am(i,s)/((1+rtms(i,r,s))*p_m(i,r,s)/pmt0(i,r,s)))**(sig_m(i,s))
    )$vxmd(i,r,s);

*	Demand for domestic varieties:
a_dd(i,r) ==
    ((vdm(i,r)/n0(i,r)) * (n(i,r))**(sig_dd(i,r)/(1-sig_dd(i,r)))
    )$(k(i) and vdm(i,r) and (not fl_h));

*	Demand for foreign varieties:
a_mm(i,r,s) ==
    ((vxmd(i,r,s)/n0(i,r)) * (n(i,r))**(sig_ff(i,s)/(1-sig_ff(i,s)))
    )$(k(i) and vxmd(i,r,s) and (not fl_h));

*	Demand from transport sector:
a_t(i,r) ==
    (vst(i,r) * (c_t / p_y(i,r))**(sig_t))$vst(i,r);

*	----------------------------------------------------------------------
*	Functions for imperfectly competitive models:

*	Elasticity of demand for Armington goods:
eod(i,r) == (eod0(i,r))$(k(i) and a0(i,r));

*	Beta:
beta(i,r,s) ==
    ((((1-rtxs(i,r,s))*p_x(i,r,s))
	/ ((1-rtxs(i,r,s))*p_x(i,r,s) + p_t*tau(i,r,s)))$(not %fl_beta%)
	+
	beta0(i,r,s)$%fl_beta%
    )$(k(i) and vxmd(i,r,s) and (not fl_cm));

*	Share of domestic goods:
s_ad(i,r) ==
    (p_ad(i,r)*ad0(i,r)*ad(i,r) / (p_a(i,r)*a0(i,r)*a(i,r))
    )$(k(i) and ad0(i,r) and (not fl_lgmc) and (not fl_cm));

*	Share of import goods:
s_am(i,r) ==
    (p_am(i,r)*am0(i,r)*am(i,r) / (p_a(i,r)*a0(i,r)*a(i,r))
    )$(k(i) and am0(i,r)  and (not fl_lgmc) and (not fl_cm));

*	Share of import from each region:
s_m(i,r,s) ==
    (((1+rtms(i,r,s))*p_m(i,r,s)*m0_(i,r,s)*m(i,r,s))
	/ (p_am(i,s)*am0(i,s)*am(i,s))
    )$(k(i) and vxmd(i,r,s) and (not fl_lgmc) and (not fl_cm));

*	Domestic supply share in total supply:
delta_d(i,r) ==
    (qd0(i,r)*q_d(i,r) / (qt0(i,r)*q_t(i,r))
    )$(fl_int and k(i) and qd0(i,r));

*	Export supply share in total supply:
delta_x(i,r,s) ==
    (qx0(i,r,s)*q_x(i,r,s) / (qt0(i,r)*q_t(i,r))
    )$(fl_int and k(i) and qx0(i,r,s));

*	----------------------------------------------------------------------
*	Functions for integrated Cournot model (model IC):

*	Conjectured ROC in p_a:
h_pa(r,i,s) ==
    (s_ad(i,s) * h_pad(r,i,s))$ad0(i,s)
    + (s_am(i,s) * h_pam0(r,i,s) * h_pam(r,i,s))$am0(i,s)
    ;
*	Conjectured ROC in p_ad:
h_pad(r,i,ss) ==
    ((1 + (n0(i,r)*n(i,r) - 1) * h_p0(r,i,r) * h_p(r,i,r))/(n0(i,r)*n(i,r))
    )$r_s(r,ss)
    +
    (h_p0(r,i,ss) * h_p(r,i,ss))$(not r_s(r,ss))
    ;
*	ROC in own total supply:
h_qtv(i,r) ==
    (delta_d(i,r) * h_qdv(i,r))$qd0(i,r)
    + sum(s$qx0(i,r,s), delta_x(i,r,s) * h_qxv(i,r,s))
    ;
*	Conjectured ROC in own domestic supply
h_qdv(i,r) ==
    - sig_dd(i,r)
    + (sig_dd(i,r) - sig_a(i,r)) * h_pad(r,i,r)
    + (sig_a(i,r) - eod(i,r)) * h_pa(r,i,r)
    ;
*	Conjectured ROC in own export supply:
h_qxv(i,r,s) ==
    - sig_ff(i,s) * beta(i,r,s)
    + (sig_ff(i,s) - sig_m(i,s)) * h_pm0(r,i,r,s) * h_pm(r,i,r,s)
    + (sig_m(i,s) - sig_a(i,s)) * h_pam0(r,i,s) * h_pam(r,i,s)
    + (sig_a(i,s) - eod(i,s)) * h_pa(r,i,s)
    ;
*	Conjectured ROC in domestic supply of rival firms
h_qd(r,i,ss) ==
    - sig_dd(i,ss) * h_p0(r,i,ss) * h_p(r,i,ss)
    + (sig_dd(i,ss) - sig_a(i,ss)) * h_pad(r,i,ss)
    + (sig_a(i,ss) - eod(i,ss)) * h_pa(r,i,ss)
    ;
*	Conjectured ROC in export supply of rival firms:
h_qx(r,i,t,s) ==
    - sig_ff(i,s) * beta(i,t,s) * h_p0(r,i,t) * h_p(r,i,t)
    + (sig_ff(i,s) - sig_m(i,s)) * h_pm0(r,i,t,s) * h_pm(r,i,t,s)
    + (sig_m(i,s) - sig_a(i,s)) * h_pam0(r,i,s) * h_pam(r,i,s)
    + (sig_a(i,s) - eod(i,s)) * h_pa(r,i,s)
    ;
*	----------------------------------------------------------------------
*	Equation definitions for model:
display "com: Equation definitions for model:";

*	----------------------------------------------------------------------
*	Imperfectly competitive sectors:

*	FOC for profit maximization of domestic supply:
e_q_d(i,r)$(vdm(i,r) and k(i))..

    (((cy0(i,r)*c_y(i,r)
	- (1-rto(i,r))*pd0(i,r)*p_d(i,r)
	*(1-mud0(i,r)*mu_d(i,r))))$(not fl_cm)
	+
	(ac0(i,r)*ac(i,r)-(1-rto(i,r))*pd0(i,r)*p_d(i,r))$fl_cm
    )$(not fl_int)
    +
    (vdm(i,r)*q_d(i,r)/n0(i,r) - a_dd(i,r)*ad(i,r))$fl_int
    =g= 0;

*	FOC for profit maximization of export supply:
e_q_x(i,r,s)$(vxmd(i,r,s) and k(i))..

    ((cy0(i,r)*c_y(i,r)
	- (1-rto(i,r))*px0(i,r,s)*p_x(i,r,s)
	* (1-mux0(i,r,s)*mu_x(i,r,s)))$(not fl_cm)
	+
	(ac0(i,r)*ac(i,r) - (1-rto(i,r))*px0(i,r,s)*p_x(i,r,s))$fl_cm
    )$(not fl_int)
    +
    (vxmd(i,r,s)*q_x(i,r,s)/n0(i,r) - a_mm(i,r,s)*m(i,r,s))$fl_int
    =g= 0;

* 	Zero profit condition:
e_n(i,r)$(vom(i,r) and k(i)) ..

    (cy0(i,r)*c_y(i,r)*qtt0(i,r)*q_tt(i,r)
	- (1-rto(i,r))
	* (pd0(i,r)*p_d(i,r)*qd0(i,r)*q_d(i,r)
	    + sum(s$qx0(i,r,s),
		px0(i,r,s)*p_x(i,r,s)*qx0(i,r,s)*q_x(i,r,s)))
    )$((not fl_cm) and (not fl_fix))
    +
    (n(i,r) - 1)$(fl_cm or fl_fix)
    =g= 0;

* 	Excess profit (valid only in fixed-number-of-firms model)
e_ep(i,r)$(vom(i,r) and k(i) and fl_fix and (not fl_cm))..

    ep(i,r) +
    cy0(i,r)*c_y(i,r)*qtt0(i,r)*q_tt(i,r)
    - (1-rto(i,r))
    * (pd0(i,r)*p_d(i,r)*qd0(i,r)*q_d(i,r)
	+ sum(s$qx0(i,r,s), px0(i,r,s)*p_x(i,r,s)*qx0(i,r,s)*q_x(i,r,s)))
    =e= 0;

*	Total output of each firm:
e_q_t(i,r)$(vom(i,r) and k(i))..

    (qt0(i,r)*q_t(i,r)
	- (qd0(i,r)*q_d(i,r) + sum(s$qx0(i,r,s), qx0(i,r,s)*q_x(i,r,s)))
    )$(not fl_int)
    +
    (cy0(i,r)*c_y(i,r) - (1-rto(i,r))*py0(i,r)*p_com(i,r)
	* (1 - mu0(i,r)*mu(i,r)))$fl_int

    =e= 0;

*	Total input (variable input + fixed inputs) of each firm:
e_q_tt(i,r)$(vom(i,r) and k(i))..

    qtt0(i,r)*q_tt(i,r)
    =e= qt0(i,r)*q_t(i,r) + fc0(i,r);

*	Average cost:
e_ac(i,r)$(k(i) and vom(i,r)) ..

    ac0(i,r)*ac(i,r) * qt0(i,r)*q_t(i,r)
    =e= cy0(i,r)*c_y(i,r) * qtt0(i,r)*q_tt(i,r);

*	Markup rate for domestic supply:
e_mu_d(i,r)$(k(i) and vdm(i,r) and (not c_ms("ic")) and (not fl_cm)) ..

*	Quantity competition model with differentiated goods
    (mud0(i,r)*mu_d(i,r)*n0(i,r)*n(i,r) - n0(i,r)*n(i,r)/sig_dd(i,r)
	- (1/sig_a(i,r) - 1/sig_dd(i,r))
	* (1 + (n0(i,r)*n(i,r) - 1) * phi0(i,r,r))
	- (1/eod(i,r) - 1/sig_a(i,r))
	* (s_ad(i,r) + (n0(i,r)*n(i,r) - s_ad(i,r)) * phi0(i,r,r))
    )$(fl_q and (not fl_h))
    +
*	Bertrand competition model
    (n0(i,r)*n(i,r)/(mud0(i,r)*mu_d(i,r)) - sig_dd(i,r)*n0(i,r)*n(i,r)
	- (sig_a(i,r) - sig_dd(i,r)
	    + s_ad(i,r) * (eod(i,r) - sig_a(i,r)))
    )$fl_p
    +
*	LGMC
    (mud0(i,r)*mu_d(i,r) - 1/sig_dd(i,r))$fl_lgmc
    +
*	Quantity competition model with homogenous goods
    (mud0(i,r)*mu_d(i,r)*n0(i,r)*n(i,r)
	- (1/sig_a(i,r)) * (1 + (n0(i,r)*n(i,r) - 1) * phi0(i,r,r))
	- (1/eod(i,r) - 1/sig_a(i,r))
	* (s_ad(i,r) + (n0(i,r)*n(i,r) - s_ad(i,r)) * phi0(i,r,r))
    )$(fl_q and fl_h)

    =e= 0;

*	Markup rate for export supply:
e_mu_xx(i,r,s)$(k(i) and vxmd(i,r,s) and (not c_ms("ic")) and (not fl_cm)) ..

*	Quantity competition model with differentiated goods
    (muxx0(i,r,s)*mu_xx(i,r,s)*n0(i,r)*n(i,r)
	- n0(i,r)*n(i,r)/sig_ff(i,s)
	- (1/sig_m(i,s) - 1/sig_ff(i,s))
	* (1 + (n0(i,r)*n(i,r) - 1)*phi0(i,r,s))
	- (1/sig_a(i,s) - 1/sig_m(i,s))
	* (s_m(i,r,s) + (n0(i,r)*n(i,r) - s_m(i,r,s))*phi0(i,r,s))
	- (1/eod(i,s) - 1/sig_a(i,s))
	* (s_am(i,s)*s_m(i,r,s)
	    + (n0(i,r)*n(i,r) - s_am(i,s)*s_m(i,r,s))*phi0(i,r,s))
    )$(fl_q and (not fl_h))
    +
*	Bertrand competition model
    (n0(i,r)*n(i,r)/(muxx0(i,r,s) * mu_xx(i,r,s))
	- sig_ff(i,s)*n0(i,r)*n(i,r)
	- (sig_m(i,s) - sig_ff(i,s)
	    + s_m(i,r,s) * (sig_a(i,s) - sig_m(i,s)
		+ s_am(i,s) * (eod(i,s) - sig_a(i,s))))
    )$fl_p
    +
*	LGMC
    (muxx0(i,r,s) * mu_xx(i,r,s) - 1/sig_ff(i,s))$fl_lgmc
    +
*	Quantity competition model with homogenous goods
    (muxx0(i,r,s)*mu_xx(i,r,s)*n0(i,r)*n(i,r)
	- (1/sig_m(i,s)) * (1 + (n0(i,r)*n(i,r) - 1)*phi0(i,r,s))
	- (1/sig_a(i,s) - 1/sig_m(i,s))
	* (s_m(i,r,s) + (n0(i,r)*n(i,r) - s_m(i,r,s))*phi0(i,r,s))
	- (1/eod(i,s) - 1/sig_a(i,s))
	* (s_am(i,s)*s_m(i,r,s)
	    + (n0(i,r)*n(i,r) - s_am(i,s)*s_m(i,r,s))*phi0(i,r,s))
    )$(fl_q and fl_h)
    =e= 0;

*	Markup rate for export supply (adjustment for transport costs):
e_mu_x(i,r,s)$(k(i) and vxmd(i,r,s) and (not c_ms("ic")) and (not fl_cm)) ..

    mux0(i,r,s) * mu_x(i,r,s) * beta(i,r,s)
    =e= muxx0(i,r,s) * mu_xx(i,r,s);

*	Common markup rates for integrated model:
e_mu(i,r)$(k(i) and vom(i,r) and fl_int) ..

*	Integrated Bertrand:
    (1/(mu0(i,r) * mu(i,r))
	- (delta_d(i,r)/(mud0(i,r)*mu_d(i,r)))$qd0(i,r)
	- sum(s$qx0(i,r,s), delta_x(i,r,s)/(mux0(i,r,s) * mu_x(i,r,s)))
    )$(fl_p and (not fl_ib_alt))
    +
    (mu0(i,r) * mu(i,r)
	- (delta_d(i,r) * (mud0(i,r)*mu_d(i,r)))$qd0(i,r)
	- sum(s$qx0(i,r,s), delta_x(i,r,s) * (mux0(i,r,s) * mu_x(i,r,s)))
    )$(fl_p and fl_ib_alt)
    +

*	Integrated Cournot:
    (mu0(i,r) * mu(i,r) * h_qtv(i,r) + 1)$fl_q

    =e= 0;

* *	Elasticity of total demand:
* e_eod_c(i,r)$(k(i) and vom(i,r) and fl_int and fl_p and fl_ib_alt) ..

*     eodc0(i,r) * eod_c(i,r) =e=
*     (delta_d(i,r) * eodd0(i,r) * eod_d(i,r))$qd0(i,r)
*     + sum(s$qx0(i,r,s), delta_x(i,r,s) * eodx0(i,r,s) * eod_x(i,r,s));

* *	Elasticity of domestic demand:
* e_eod_d(i,r)$(k(i) and qd0(i,r) and fl_int and fl_p and fl_ib_alt) ..

*     eodd0(i,r) * eod_d(i,r) * mud0(i,r) * mu_d(i,r) =e= 1;

*	Elasticity of total demand:
* e_eod_x(i,r,s)$(k(i) and qx0(i,r,s) and fl_int and fl_p and fl_ib_alt) ..

*     eodx0(i,r,s) * eod_x(i,r,s) * mux0(i,r,s) * mu_x(i,r,s) =e= 1;

*	----------------------------------------------------------------------
*	Equations for integrated market Cournot model:

*	Conjectured ROC in prices of rival firms (Cournot conjecture):
e_h_p(r,i,t)$(k(i) and vom(i,t) and c_ms("ic")) ..

    (delta_d(i,t)*h_qd(r,i,t))$qd0(i,t)
    + sum(s$qx0(i,t,s), delta_x(i,t,s)*h_qx(r,i,t,s))
    =e= 0;

*	Conjectured ROC in p_m:
e_h_pm(r,i,t,s)$(k(i) and c_ms("ic") and vxmd(i,t,s)) ..

    h_pm0(r,i,t,s)*h_pm(r,i,t,s) =e=
    ((beta(i,r,s)
	+ (n0(i,r)*n(i,r)-1)*beta(i,r,s)*h_p0(r,i,r)*h_p(r,i,r))
	/(n0(i,r)*n(i,r))
    )$r_s(r,t)
    +
    (beta(i,t,s)*h_p0(r,i,t)*h_p(r,i,t))$(not r_s(r,t));

*	Conjectured ROC in p_am:
e_h_pam(r,i,s)$(k(i) and am0(i,s) and c_ms("ic")) ..

    h_pam0(r,i,s)*h_pam(r,i,s) =e=
    sum(tt$vxmd(i,tt,s), s_m(i,tt,s)*h_pm0(r,i,tt,s)*h_pm(r,i,tt,s));

*	----------------------------------------------------------------------
*	Zero profit conditions.

*	Production:
e_y(i,r)$(l(i) and vom(i,r)) ..

    c_y(i,r) =g= (1-rto(i,r)) * p_y(i,r) / po0(i,r);

*	Production of inputs for transport sector in K
e_y_xt(i,r)$(k(i) and sh_trns(i,r)) ..

    c_y(i,r) =g= (1-rto(i,r)) * p_y(i,r) / po0(i,r);

*	Armington aggregation:
e_a(i,r)$a0(i,r) ..

    c_a(i,r) =g= p_a(i,r);

*	Domestic varieties:
e_ad(i,r)$(k(i) and ad0(i,r)) ..

    (c_ad(i,r) - p_ad(i,r))$(not fl_h)
    +
    (ad0(i,r)*ad(i,r) - a_ad(i,r) * a(i,r))$fl_h
    =g= 0;

*	Import aggregation:
e_am(i,r)$vim(i,r) ..

    c_am(i,r) =g= p_am(i,r);

*	Foreign varieties (import):
e_m(i,r,s)$vxmd(i,r,s) ..

    ((c_m(i,r,s) - p_m(i,r,s))$(not fl_h)
	+ (vxmd(i,r,s)*m(i,r,s) - a_m(i,r,s) * am(i,s))$fl_h)$k(i)
    +
    (vxmd(i,r,s)*m(i,r,s) - a_m(i,r,s) * am(i,s))$l(i) =e= 0;

*	Utility:
e_u(r) ..

    c_u(r) =g= p_u(r);

*	Transport sector:
e_yt ..

    c_t =g= p_t;

*	----------------------------------------------------------------------
*	Market clearing condtions.

*	Market of outputs:
e_p_y(i,r)$(vom(i,r) and (l(i) or (k(i) and sh_trns(i,r)))) ..

    (vom(i,r) * y(i,r)
	- sum(s$vxmd(i,r,s), a_m(i,r,s) * am(i,s))
	- (a_t(i,r) * yt)$vst(i,r)
	- (a_ad(i,r) * a(i,r))$ad0(i,r)
	- (scale_i(r) * inv(r))$cgd(i)
    )$l(i)
    +
    (sh_trns(i,r) * vom(i,r) * y_xt(i,r) - a_t(i,r) * yt
    )$(k(i) and sh_trns(i,r))
    =g= 0;

*	Outputs in the integrated market.
e_p_com(i,r)$(vom(i,r) and k(i) and fl_int) ..

    (1-sh_trns(i,r))*vom(i,r)*q_t(i,r)/n0(i,r)
    - (a_dd(i,r) * ad(i,r))$vdm(i,r)
    - sum(s$vxmd(i,r,s), a_mm(i,r,s) * m(i,r,s))
    =g= 0;

*	Market for domestic output of imperfectly competititve sectors:
e_p_d(i,r)$(k(i) and vdm(i,r)) ..

    ((vdm(i,r)*q_d(i,r)/n0(i,r) - a_dd(i,r) * ad(i,r))$(not fl_h)
	+ (pd0(i,r)*n0(i,r)*n(i,r)*qd0(i,r)*q_d(i,r) - ad0(i,r)*ad(i,r))$fl_h
    )$(not fl_int)
    +
    (p_d(i,r) - p_com(i,r))$fl_int
    =g= 0;

*	Market for export of imperfectly competititve sectors:
e_p_x(i,r,s)$(k(i) and vxmd(i,r,s)) ..

    ((vxmd(i,r,s)*q_x(i,r,s)/n0(i,r) - a_mm(i,r,s) * m(i,r,s))$(not fl_h)
	+
	(p_m(i,r,s) - p_x_(i,r,s))$fl_h
    )$(not fl_int)
    +
    (p_x(i,r,s) - p_com(i,r))$fl_int
    =g= 0;

*	Domestic goods:
e_p_ad(i,r)$(k(i) and ad0(i,r)) ..

    (ad0(i,r) * ad(i,r) - a_ad(i,r) * a(i,r))$(not fl_h)
    +
    (p_ad(i,r) - p_d(i,r))$fl_h
    =g= 0;

*	Market for aggregate import:
e_p_am(i,r)$am0(i,r) ..

    am0(i,r)*am(i,r) =g= a_am(i,r)*a(i,r);

*	Market for import:
e_p_m(i,r,s)$vxmd(i,r,s) ..

    ((m0(i,r,s) * m(i,r,s) - a_m(i,r,s) * am(i,s))$(not fl_h)
	+ (px0(i,r,s)*n0(i,r)*n(i,r)*qx0(i,r,s)*q_x(i,r,s)
	    - m0(i,r,s) * m(i,r,s))$fl_h
    )$k(i)
    +
    (p_m(i,r,s) - p_x_(i,r,s))$l(i) =e= 0
    ;
*	Market of transport services:
e_p_t ..

    vt * yt =g=
    sum((l,r,s)$vxmd(l,r,s), tau(l,r,s)*a_m(l,r,s)*am(l,s))
    +
    sum((k,r,s)$vxmd(k,r,s),
	n0(k,r)*n(k,r)*tau(k,r,s)*px0(k,r,s)*qx0(k,r,s)*q_x(k,r,s));

*	Market of Armington goods:
e_p_a(i,r)$a0(i,r) ..

    a0(i,r) * a(i,r) =g=
    sum(l$v_fm(i,l,r), v_fm(i,l,r) * y(l,r))
    + sum(k$sh_trns(k,r), sh_trns(k,r) * v_fm(i,k,r) * y_xt(k,r))
    + sum(k$v_fm(i,k,r), (1 - sh_trns(k,r)) * v_fm(i,k,r) * n(k,r)*q_tt(k,r))
    + (a_c(i,r) * u(r))$v_cm(i,r)
    ;
*	Market for primary factors:
e_p_f(f,r) ..

    scale_f(f,r) * evom(f,r)
    =g=  sum(l$vfm(f,l,r), a_f(f,l,r) * y(l,r))
    + sum(k$sh_trns(k,r), sh_trns(k,r) * a_f(f,k,r) * y_xt(k,r))
    + sum(k$vfm(f,k,r), (1 - sh_trns(k,r)) * a_f(f,k,r)*n(k,r)*q_tt(k,r));

*	Market for utility:
e_p_u(r) ..

    inc_ra(r) =g= p_u(r) * vcm(r) * u(r);

*	----------------------------------------------------------------------
*	Regional income
e_inc_ra(r) ..

    inc_ra(r) =e=

*	Factor income:
    sum(f$evom(f,r), p_f(f,r) * scale_f(f,r) * evom(f,r))

*	Ouput tax:
    + sum(l$vom(l,r), rto(l,r)*p_y(l,r)*vom(l,r)*y(l,r))

    + sum(k$sh_trns(k,r), rto(k,r)*p_y(k,r)*vom(k,r)*y_xt(k,r)*sh_trns(k,r))

    + sum(k$vom(k,r), rto(k,r)
	*n(k,r) * (p_d(k,r)*vdm(k,r)*q_d(k,r)
	    + sum(s, p_x(k,r,s)*vxmd(k,r,s)*q_x(k,r,s))))

*	Intermediate tax:
    + sum((j,l)$v_fm(j,l,r), rti(j,l,r)*p_a(j,r)*v_fm(j,l,r)*y(l,r))

    + sum((k,j)$v_fm(j,k,r),
	rti(j,k,r)*p_a(j,r)*v_fm(j,k,r)
	* (sh_trns(k,r)*y_xt(k,r) + (1-sh_trns(k,r))*n(k,r)*q_tt(k,r)))

*	Factor tax:
    + sum((f,l)$vfm(f,l,r), rtf(f,l,r)*p_f(f,r)*a_f(f,l,r)*y(l,r))

    + sum((f,k)$vfm(f,k,r),
	rtf(f,k,r)*p_f(f,r)*a_f(f,k,r)
	* (sh_trns(k,r)*y_xt(k,r) + (1-sh_trns(k,r))*n(k,r)*q_tt(k,r)))

*	Export subsidies:
    - sum((l,s)$vxmd(l,r,s), rtxs(l,r,s)*p_y(l,r)*a_m(l,r,s)*am(l,s))

    - sum((k,s)$vxmd(k,r,s),
	rtxs(k,r,s)*n0(k,r)*n(k,r)*px0(k,r,s)*p_x(k,r,s)*qx0(k,r,s)*q_x(k,r,s))

*	Tariff:
    + sum((l,s)$vxmd(l,s,r),
	rtms(l,s,r)*((1-rtxs(l,s,r))*p_y(l,s) + p_t*tau(l,s,r))*a_m(l,s,r)*am(l,r))

    + sum((k,s)$vxmd(k,s,r),
	rtms(k,s,r)*((1-rtxs(k,s,r))*p_x(k,s,r) + p_t*tau(k,s,r))
	*px0(k,s,r)*n0(k,s)*n(k,s)*qx0(k,s,r)*q_x(k,s,r))

*	Consumption tax:
    + sum(i$v_cm(i,r), rtc(i,r)*p_a(i,r)*a_c(i,r)*u(r))

*	Excess profit:
    + (sum(k, n0(k,r)*n(k,r)*ep(k,r)))$fl_fix

    - scale_i(r) * p_inv(r) * inv(r)

    + sum(rnum, p_u(rnum)) * scale_bop(r) * vb(r)
    ;

*	Dummy objective variable for NLP model:
e_dum_obj ..

    dum_obj =e= 1;

*	----------------------------------------------------------------------
*	Model declaration:
display "com: Model declaration:";

model gtap_trade_lib_mcp GTAPinGAMS MCP model /

      e_q_d.q_d, e_q_x.q_x, e_n.n, e_q_tt.q_tt, e_q_t.q_t, e_mu_d.mu_d,
      e_mu_xx.mu_xx, e_mu_x.mu_x, e_mu.mu, e_y.y, e_y_xt.y_xt, e_a.a, e_ad.ad,
      e_am.am, e_m.m, e_u.u, e_yt.yt, e_p_y.p_y, e_p_com.p_com, e_p_d.p_d,
      e_p_x.p_x, e_p_ad.p_ad, e_p_am.p_am, e_p_m.p_m, e_p_t.p_t, e_p_a.p_a,
      e_p_f.p_f, e_p_u.p_u, e_inc_ra.inc_ra, e_ac.ac, e_ep.ep,

*       e_eod_d.eod_d,
*       e_eod_x.eod_x, e_eod_c.eod_c,

      e_h_p.h_p, e_h_pam.h_pam, e_h_pm.h_pm

      /;

model gtap_trade_lib_nlp GTAPinGAMS NLP model /

      e_q_d, e_q_x, e_n, e_q_tt, e_q_t, e_mu_d, e_mu_xx, e_mu_x, e_mu, e_y,
      e_a, e_ad, e_am, e_m, e_u, e_yt, e_p_y, e_p_com, e_y_xt, e_p_d, e_p_x,
      e_p_ad, e_p_am, e_p_m, e_p_t, e_p_a, e_p_f, e_p_u, e_inc_ra, e_ac, e_ep,

*     e_eod_d, e_eod_x, e_eod_c,

      e_h_p, e_h_pam, e_h_pm,

      e_dum_obj
      /;

* --------------------
* Local Variables:
* mode: gams
* fill-column: 78
* tab-width: 8
* End:
